C
Code by Michael Kupferschmid
C
C     This program reads lines from standard in, rearranges the
C     columns, and writes them to standard out.
C
C     unit  use              $?  meaning
C     ----  --------------   --  -------------
C       0   error messages    0  all went well
C       5   input lines       1  could not parse parameter string
C       6   output lines      2  too many output columns specified
C                             3  no output columns specified
C                             4  bogus output column specified
C
C     If no output columns are given (RC=3) the program writes one
C     line of usage information.  If a requested output column does
C     not occur in the input line, a blank is output in that column. 
C
C     variable  meaning
C     --------  -------
C     ARGLST    routine parses run-line argument string
C     EXIT      system routine stops with a return code
C     INPUT     line read from standard input
C     K         index on the characters of a string
C     L         length of a string
C     LENGTH    function returns index of last nonblank in a string
C     LIST      input columns to assemble into output line
C     N         length of output line
C     OUTPUT    line written to standard output
C     RC        return code; see table above
C
C     allow lines as long as 256 characters
      CHARACTER*1 INPUT(256),OUTPUT(256)
      INTEGER*4 LIST(256),RC
C
C ------------------------------------------------------------------
C
C     examine the run-line argument string
      CALL ARGLST(LIST,256, N,RC)
      IF(RC.EQ.0) GO TO 1
C
C     report errors
      IF(RC.EQ.3) WRITE(0,901)
  901 FORMAT('usage: colcut 2 3 4-6 7-3 12 1 < input > output')
      IF(RC.EQ.2) WRITE(0,902)
  902 FORMAT('colcut: too many columns specified')
      IF(RC.EQ.1) WRITE(0,903)
  903 FORMAT('colcut: could not parse run-line parameter string')
      CALL EXIT(RC)
C
C     check that the column numbers make sense
    1 DO 2 K=1,N
           IF(LIST(K).LT.0 .OR. LIST(K).GT.256) THEN
              WRITE(0,904)
  904         FORMAT('colcut: bogus output column specified')
              CALL EXIT(4)
           ENDIF
    2 CONTINUE
C
C     read the next input line
    3 READ(5,900,END=4) INPUT
  900 FORMAT(256A1)
      L=LENGTH(INPUT,256)
C
C     construct the output line
      DO 5 K=1,N
           IF(LIST(K).LE.L) THEN
              OUTPUT(K)=INPUT(LIST(K))
           ELSE
              OUTPUT(K)=' '
           ENDIF
    5 CONTINUE
C
C     write the resulting output line
      WRITE(6,900) (OUTPUT(K),K=1,N)
C
C     continue until EOF
      GO TO 3
    4 CALL EXIT(0)
      END
