C
Code by Michael Kupferschmid
C
C     This program lines up words in columns, wrapping output lines
C     if necessary, to list them all within the display width.
C
C     unit  use                  return code  meaning
C     ----  ------------------   -----------  ----------------------
C     5     list of words             0       all went well
C     6     display of columns        1       character limit met
C     13    debugging output          2       word limit met
C                                     3       column limit met
C     variable  meaning
C     --------  -------
C     BTD       routine converts an INTEGER*4 to numerals
C     CHARS     the words as columns of characters
C     CW        column widths in characters
C     DEBUG     T => write debugging output
C     DTB       routine converts numerals to an INTEGER*4
C     EXIT      system routine stops program with a return code
C     FMT       object-time format for reading input bytestream
C     GETENV    unix routine gets an environment variable
C     I         index on the words
C     ITAB      hex code for tab character
C     J         index on the columns in a row
C     K         index on the characters of a string
C     KK        index on the characters in an output line
C     KMAX      maximum characters in next word
C     L         length of a string
C     LENGTH    function returns index of last nonblank in a string
C     LINE      input stream of bytes
C     LS        the lengths of the words
C     LSMIN     length of the shortest word
C     M         number of output lines (columns) in RASTER
C     MAXCH     maximum number of input characters allowed
C     MAXM      limit on number of output lines
C     MAXRW     maximum length of an output line
C     MAXWD     maximum number of words allowed
C     MIN0      Fortran function returns smaller of INTEGER*4s
C     NC        trial number of columns
C     NCMAX     largest number of columns it is necessary to try
C     NCUSE     number of columns used
C     NW        number of words read in
C     R         number of a RASTER row being written out
C     RASTER    columns are lines to be written out
C     RC        return code; 0 => all went well
C     SCREEN    number of characters that will fit on the screen
C     SHIFTL    routine removes leading blanks
C     TAB       hex code for tab character
C     W         width of a trial row
C     WIDTH     SCREEN as numerals
C     WORD      a word read in
C     WORDS     the list of words
C
C     adjust this parameter to change the output lines allowed
      PARAMETER(MAXM=100)
C
C     the window is usually 98 characters wide
C     so plan to write 4 words of 24 characters (=96) on each line
      PARAMETER(MAXWD=4*MAXM)
C
C     then the input stream and output raster must be this big
      PARAMETER(MAXRW=4*24,MAXCH=MAXRW*MAXM)
      CHARACTER*1 LINE(MAXCH+1),RASTER(MAXRW,MAXM)
      EQUIVALENCE(LINE,RASTER)
      CHARACTER*1 FMT(8)/'(',' ',' ',' ',' ','A','1',')'/
C
C     and the word list must be this big
      CHARACTER*24 WORDS(MAXWD),WORD
      CHARACTER*1 CHARS(24,MAXWD)/MAXCH*' '/
      EQUIVALENCE(WORDS,CHARS)
      INTEGER*4 LS(MAXWD)
C
C     other variables
      CHARACTER*1 TAB
      INTEGER*1 ITAB/Z'09'/
      EQUIVALENCE(TAB,ITAB)
      CHARACTER*2 WIDTH
      INTEGER*4 CW(96)/96*0/,R,RC,SCREEN,W
      LOGICAL*4 DEBUG/.FALSE./
C
C ------------------------------------------------------------------
C
C     report dimensions for debugging
      IF(DEBUG) WRITE(13,991) MAXM,MAXWD,MAXRW,MAXCH
  991 FORMAT('MAXM=',I4,' MAXWD=',I4,' MAXRW=',I4,' MAXCH=',I4)
C
C     the window might be narrower than the assumed 4*24 characters
      CALL GETENV('COLUMNS',WIDTH)
      CALL DTB(WIDTH,2, SCREEN,RC)
      IF(RC.NE.0 .OR. SCREEN.LT.24 .OR. SCREEN.GT.96) SCREEN=96
      IF(DEBUG) WRITE(13,992) WIDTH,SCREEN,RC
  992 FORMAT('WIDTH=',A2,' SCREEN=',I2,' RC=',I1)
C
C     get the next input line, possibly with several words on it
      CALL BTD(MAXCH+1,FMT(2),4,RC)
      NW=1
    2 READ(5,FMT,END=1) LINE
      IF(LINE(MAXCH+1).NE.' ') THEN
         WRITE(0,901) MAXCH
  901    FORMAT('character limit of',I5,' reached' )
         CALL EXIT(1)
      ENDIF
C
C     translate tabs to blanks
      L=LENGTH(LINE,MAXCH)
      IF(L.EQ.0) GO TO 2
      DO 3 K=1,L
           IF(LINE(K).EQ.TAB) LINE(K)=' '
    3 CONTINUE
      IF(DEBUG) WRITE(13,FMT) (LINE(K),K=1,L)
C
C     parse out the words into CHARS and hence WORDS
    5 IF(L.EQ.0) GO TO 2
      KMAX=MIN0(L,24)
      DO 4 K=1,KMAX
           IF(LINE(K).NE.' ') THEN
              CHARS(K,NW)=LINE(K)
              LINE(K)=' '
           ELSE
              CALL SHIFTL(LINE,L)
              L=LENGTH(LINE,L)
              NW=NW+1
              IF(NW.GT.MAXWD) THEN
                 WRITE(0,902) MAXWD
  902            FORMAT('word limit of',I5,' reached')
                 CALL EXIT(2)
              ENDIF
              GO TO 5
           ENDIF
    4 CONTINUE
      NW=NW+1
      IF(NW.GT.MAXWD) THEN
         WRITE(0,902) MAXWD
         CALL EXIT(2)
      ENDIF
      GO TO 2
C
C     find their lengths
    1 IF(NW.EQ.1) STOP
      NW=NW-1
      IF(DEBUG) WRITE(13,993) NW
  993 FORMAT('NW=',I3)
      LSMIN=SCREEN
      DO 6 I=1,NW
           L=LENGTH(WORDS(I),24)
           LS(I)=L
           IF(L.LT.LSMIN) LSMIN=L
    6 CONTINUE
      IF(DEBUG) WRITE(13,994) NW,LSMIN
  994 FORMAT('NW=',I3,' LSMIN=',I2)
C
C     find the widest arrangement that fits
      NCMAX=SCREEN/(LSMIN+1)
      DO 7 NC=NCMAX,1,-1
C          to start with no words have been placed
           I=0
C
C          and the columns are empty
           DO 8 J=1,NCMAX
                CW(J)=0
    8      CONTINUE
           J=0
C
C          are the words used up?
   12      IF(I.EQ.NW) THEN
              W=CW(1)
              DO 9 J=2,NC
                   W=W+1+CW(J)
    9         CONTINUE
              IF(W.GT.SCREEN) GO TO 10
              NCUSE=NC
              GO TO 11
           ENDIF
C
C          is the line full?
           IF(J.EQ.NC) J=0
C
C          no; put the next word in the next spot on this line
           I=I+1
           J=J+1
           IF(LS(I).GT.CW(J)) CW(J)=LS(I)
           GO TO 12
C
C          watch what is happening
   10      IF(DEBUG) WRITE(13,995) NC,W,(CW(J),J=1,NCMAX)
  995      FORMAT('NC=',I2,' W=',I3,' CW=',50(I2,1X))
    7 CONTINUE
   11 IF(DEBUG) WRITE(13,996) NCUSE,W,(CW(J),J=1,NCMAX)
  996 FORMAT('NC=',I2,' W=',I3,' CW=',50(I2,1X))
C
C     found the right arrangement; use it
      M=1
      J=1
      I=1
      KK=0
   15 DO 13 K=1,CW(J)
            KK=KK+1
            RASTER(KK,M)=CHARS(K,I)
   13 CONTINUE
      IF(I.EQ.NW) GO TO 14
      I=I+1
      IF(J.EQ.NCUSE) THEN
         M=M+1
         IF(M.GT.MAXM) THEN
            WRITE(0,903) MAXM
  903       FORMAT('output line limit of',I5,' reached')
            CALL EXIT(3)
         ENDIF
         J=0
         KK=-1
      ENDIF
      J=J+1
      KK=KK+1
      GO TO 15
C
C     write out the raster
   14 DO 16 R=1,M
            L=LENGTH(RASTER(1,R),96)
            WRITE(6,904) (RASTER(K,R),K=1,L)
  904       FORMAT(96A1)
   16 CONTINUE
      STOP
      END
