C
Code by Michael Kupferschmid
C
C     This program reads lines from standard-in, justifies
C     each blank-separated token within its output column,
C     and writes the result to standard-out.
C
C     variable  meaning
C     --------  -------
C     ARG       a command-line parameter
C     DTB       routine converts numerals to an INTEGER*4
C     EXIT      system routine stops program with return code
C     GETARG    unix routine returns a command-line parameter
C     I         index on command-line parameters
C     IARGC     unix function for number of command-line parameters
C     INPUT     an input line
C     JUST      tells how to justify input tokens
C     K         index on the characters of a string
C     KMAX      maximum length of input and output lines
C     LENGTH    function returns index of last nonblank in a string
C     LIN       length of INPUT
C     LOUT      length of OUTPUT
C     N         number of columns formatted in output lines
C     OUTPUT    an output line
C     RC        return code from DTB; 0 => all went well
C     SHFTKN    routine removes leftmost tokens
C     SHIFTL    routine left-justifies text in a string
C     SHIFTR    routine right-justifies text in a string
C     W         total output columns required
C     WIDE      widths of output columns
C
      CHARACTER*3 ARG
      PARAMETER(KMAX=120)
C     the 900 FORMAT also uses this number
      CHARACTER*1 INPUT(KMAX),OUTPUT(KMAX),JUST(KMAX/2)
      INTEGER*4 WIDE(KMAX/2),W,RC
C
C ------------------------------------------------------------------
C
C     find out about the output columns
      N=IARGC()
      IF(N.EQ.0) THEN
         WRITE(0,901)
  901    FORMAT('usage: colign R10 L20 ...')
         CALL EXIT(0)
      ENDIF
      W=N-1
      DO 1 I=1,N
           CALL GETARG(I,ARG)
C
C          remember the justification
           JUST(I)=' '
           IF(ARG(1:1).EQ.'l' .OR. ARG(1:1).EQ.'L') JUST(I)='L'
           IF(ARG(1:1).EQ.'r' .OR. ARG(1:1).EQ.'R') JUST(I)='R'
           IF(JUST(I).EQ.' ') THEN
              WRITE(0,902) I
  902         FORMAT('justification ',I2,' must be L or R')
              CALL EXIT(1)
           ENDIF
C
C          give a 1-digit width a leading zero
           IF(ARG(3:3).EQ.' ') THEN
              ARG(3:3)=ARG(2:2)
              ARG(2:2)='0'
           ENDIF
C
C          remember the column width
           CALL DTB(ARG(2:2),2,WIDE(I),RC)
           IF(RC.NE.0) THEN
              WRITE(0,903) I,ARG(2:2),ARG(3:3)
  903         FORMAT('width ',I2,' = ',A1,A1,' must be numerals')
              CALL EXIT(1)
           ENDIF
           W=W+WIDE(I)
    1 CONTINUE
      IF(W .GT. KMAX) THEN
         WRITE(6,904) W
  904    FORMAT('increase KMAX to at least ',I3)
         CALL EXIT(2)
      ENDIF
C
C     get the next input line
    3 READ(5,900,END=2) INPUT
  900 FORMAT(120A1)
      CALL SHIFTL(INPUT,KMAX)
      LIN=LENGTH(INPUT,KMAX)
      IF(LIN.EQ.0) THEN
         WRITE(6,900) ' '
         GO TO 3
      ENDIF
C
C     start with a blank output line
      DO 4 K=1,KMAX
           OUTPUT(K)=' '
    4 CONTINUE
      LOUT=0
      I=0
C
C     extract the next token and justify it
    7 I=I+1
      DO 5 K=1,LIN
           IF(INPUT(K).EQ.' ') GO TO 6
           IF(K.GT.WIDE(I)) THEN
              WRITE(6,905) I,WIDE(I)
  905         FORMAT('token ',I2,' width exceeds ',I2)
              CALL EXIT(3)
           ENDIF
           OUTPUT(LOUT+K)=INPUT(K)
    5 CONTINUE
    6 IF(JUST(I).EQ.'L') CALL SHIFTL(OUTPUT(LOUT+1),WIDE(I))
      IF(JUST(I).EQ.'R') CALL SHIFTR(OUTPUT(LOUT+1),WIDE(I))
      LOUT=LOUT+WIDE(I)+1
C
C     go for the next token
      CALL SHFTKN(INPUT,LIN,1,RC)
      LIN=LENGTH(INPUT,LIN)
      IF(I.LT.N .AND. LIN.GT.0) GO TO 7
C
C     write the formatted result and go for the next input line
      WRITE(6,900) (OUTPUT(K),K=1,LOUT),(INPUT(K),K=1,LIN)
      GO TO 3
C
C     the end of the input stream was reached
    2 CALL EXIT(0)
      END
