C
Code by Michael Kupferschmid
C
C     This program compares the member lists of two tar.gz archives.
C     The archive X member list is generated using tar tvf X.tar.gz.
C
C     RC  meaning
C     --  -------
C      0  member names and byte counts match
C      1  member names match but some byte count differs
C      2  member names differ
C      3  the lists are of different lengths
C      4  an archive has too many members
C
C     variable  meaning
C     --------  -------
C     BLANK     the character ' '
C     DONE      T => done blanking out 
C     EXIT      system routine stops program with a return code
C     GETFIL    routine attaches a file for a given purpose
C     I         index on archive members
C     K         index on the characters of a name
C     L         length of an archive member name
C     LENGTH    function returns index of last nonblank in a string
C     LIMIT     maximum number of archive members allowed
C     M         number of members found in list A
C     MAX0      Fortran function returns larger of INTEGER*4s
C     N         number of members found in list B
C     NAMCHR    NAME as separate characters
C     NAME      a member name
C     NAMEA     names of members in archive A
C     NAMEB     names of members in archive B
C     RC        return code; see above
C     SHIFTL    routine removes leading blanks from a string
C     SIZE      a member byte count
C     SIZEA     byte counts of members in archive A
C     SIZEB     byte counts of members in archive B
C
C     prepare to remember the names and sizes
      PARAMETER(LIMIT=1024)
      CHARACTER*64 NAMEA(LIMIT),NAMEB(LIMIT),NAME
        CHARACTER*1 NAMCHR(64)
        EQUIVALENCE(NAME,NAMCHR)
      INTEGER*4 SIZEA(LIMIT),SIZEB(LIMIT),SIZE,RC
      CHARACTER*1 BLANK/' '/
      LOGICAL*4 DONE
C
C ------------------------------------------------------------------
C
C     read the sizes and names of the members of archive A
      CALL GETFIL('list A',6,' ',0,1,1)
      M=0
    2 M=M+1
      IF(M.GT.LIMIT) THEN
         WRITE(0,991)
 991     FORMAT('archive A has too many members')
         CALL EXIT(4)
      ENDIF
      READ(1,901,END=1) SIZE,NAME
  901 FORMAT(T21,I10,T49,A64)
      SIZEA(M)=SIZE
      DONE=.FALSE.
      DO 3 K=1,64
           IF(NAMCHR(K).EQ.'/') DONE=.TRUE.
           NAMCHR(K)=BLANK
           IF(DONE) THEN
              CALL SHIFTL(NAMCHR,64)
              NAMEA(M)=NAME
              GO TO 2
           ENDIF
    3 CONTINUE
C
C     read the sizes and names of the members of archive B
    1 CALL GETFIL('list B',6,' ',0,2,1)
      N=0
    4 N=N+1
      IF(N.GT.LIMIT) THEN
         WRITE(0,992)
 992     FORMAT('archive B has too many members')
         CALL EXIT(4)
      ENDIF
      READ(2,901,END=5) SIZE,NAME
      SIZEB(N)=SIZE
      DONE=.FALSE.
      DO 6 K=1,64
           IF(NAMCHR(K).EQ.'/') DONE=.TRUE.
           NAMCHR(K)=BLANK
           IF(DONE) THEN
              CALL SHIFTL(NAMCHR,64)
              NAMEB(N)=NAME
              GO TO 4
           ENDIF
    6 CONTINUE
C
C     do the archives contain the same number of members?
    5 IF(N.NE.M) CALL EXIT(3)
C
C     do the members have the same names?
      DO 7 I=1,N
           IF(NAMEA(I).NE.NAMEB(I)) CALL EXIT(2)
    7 CONTINUE
C
C     report the names of any members having different sizes
      RC=0
      DO 8 I=1,N
           IF(SIZEA(I).NE.SIZEB(I)) THEN
              RC=1
              NAME=NAMEA(I)
              L=LENGTH(NAMCHR,64)
              WRITE(6,902) (NAMCHR(K),K=1,L)
  902         FORMAT(64A1)
           ENDIF
    8 CONTINUE
      CALL EXIT(RC)
      END
