C
Code by Michael Kupferschmid
C
C     This program updates an annotated list of file names.
C
C     unit  assignment   contents
C     ----  -----------  --------
C       7   /tmp/=files  list of filenames currently in directory
C       8   =files.old   previous list to update
C       9   =files       updated list
C
C     variable  quantity
C     --------  --------
C     DATIME    routine returns date and time
C     FILES     list of filenames in the catalog
C     FIRST     T => this is the first unknown name
C     FOUND     T => this current name is in =files.old
C     GLIMIT    maximum characters of deleted files
C     GONE      list of files no longer present
C     K         index on the characters of a string
C     KMAX      number of filename characters to copy
C     KREM      index on the characters of GONE
C     L         length of TEXT
C     LENGTH    function returns index of last nonblank in a string
C     LIST      names of files listed in =files
C     LSTMOD    routine returns day-date-time of last file mod
C     MAX0      Fortran function for larger of INTEGER*4s
C     MIN0      Fortran function returns smaller of INTEGER*4s
C     N         index on the filenames in =files or the catalog
C     NCAT      number of filenames in the catalog
C     NLIMIT    maximum number of filenames allowed
C     NLM       NLST-1
C     NLST      number of old files listed in =files
C     NN        second index on filenames for checking duplicates
C     NONEW     number of filenames added to the list
C     NOREM     number of filenames removed from the list
C     NOW       Www dd Mmm yy hh:mm:ss
C     NP        N+1
C     OK        T => =files already exists
C     RC        return code from a SYSTEM call
C     SYSTEM    system routine issues a shell command
C     TEXT      a line of text
C
C     prepare to table the filenames in the catalog and in =files
      PARAMETER(NLIMIT=2000)
      CHARACTER*1 FILES(24,NLIMIT),LIST(24,NLIMIT)
      LOGICAL*4 FOUND(NLIMIT)
C
C     prepare to list the files that disappeared
      CHARACTER*1 WHEN(22)
      INTEGER*4 GLIMIT
      PARAMETER(GLIMIT=24000)
      CHARACTER*1 GONE(GLIMIT)/23997*' ','.','.','.'/
C
C     other variables
      CHARACTER*1 NOW(22),TEXT(68)
      INTEGER*4 SYSTEM,RC
      LOGICAL*4 FIRST,OK
C
C ------------------------------------------------------------------
C
C     if there is no =files file, start one
      INQUIRE(FILE='=files',EXIST=OK)
      IF(.NOT.OK) THEN
         OPEN(UNIT=9,FILE='=files',STATUS='NEW')
         WRITE(9,901)
  901    FORMAT('directory maintenance'/68('-')/
     ;          ' ./                       pointer here'/
     ;          ' ../                      pointer up'/
     ;          ' =files                   this table of contents')
         CLOSE(UNIT=9)
      ENDIF
C
C     table the names of the files in this directory, except .swp
      RC=SYSTEM('/bin/rm -f /tmp/=files')
      RC=SYSTEM('/bin/ls -aF > /tmp/=files')
      OPEN(7,FILE='/tmp/=files',STATUS='OLD')
      NCAT=0
      DO 1 N=1,NLIMIT
           READ(7,902,END=2) TEXT
  902      FORMAT(68A1)
           L=LENGTH(TEXT,68)
C
C          if the file is a .swp file, skip it
           IF(L.LE.4) GO TO 3
           IF(TEXT(L-3).NE.'.') GO TO 3
           IF(TEXT(L-2).NE.'s') GO TO 3
           IF(TEXT(L-1).NE.'w') GO TO 3
           IF(TEXT(L  ).NE.'p') GO TO 3
           GO TO 1
C
C          count the file
    3      NCAT=NCAT+1
           FOUND(NCAT)=.FALSE.
C
C          truncate long names to fit in the list
           KMAX=MIN0(L,24)
           DO 4 K=1,24
                IF(K.LE.KMAX) FILES(K,NCAT)=TEXT(K)
                IF(K.GT.KMAX) FILES(K,NCAT)=' '
    4      CONTINUE
           IF(L.GT.24) FILES(24,NCAT)='>'
    1 CONTINUE
      WRITE(6,903) NLIMIT
  903 FORMAT('Too many (>',I4,') files; sorry.')
      STOP
C
C     move =files to =files.old and prepare to read it
    2 CALL LSTMOD('=files',6, WHEN,RC)
      CALL DATIME(NOW)
      RC=SYSTEM('/bin/rm -f =files.old')
      RC=SYSTEM('/bin/mv -f =files =files.old')
      OPEN(8,FILE='=files.old')
C
C     prepare to write a new =files file
      OPEN(9,FILE='=files',STATUS='NEW')
      KREM=0
      NOREM=0
      NONEW=0
C
C     copy lines from =files.old to =files omitting names now gone
C          to begin with no names have been copied
           NLST=0
C
C          read the next line of the old =files file
   11      READ(8,902,END=5) TEXT
           L=MAX0(1,LENGTH(TEXT,68))
C
C          copy the line if all blank or not blank in col 1
           IF(L.EQ.1) GO TO 6
           IF(TEXT(1).NE.' ') GO TO 6
C
C          the line gives a filename; is the file now present?
           DO 7 N=1,NCAT
              DO 8 K=1,24
C                   old name     new name
                 IF(TEXT(K+1).NE.FILES(K,N)) THEN
C                   names differ; is it only because one ends in *?
                    IF(TEXT(K+1).EQ.'*'.AND.FILES(K,N).EQ.' '  .OR.
     ;                 TEXT(K+1).EQ.' '.AND.FILES(K,N).EQ.'*') THEN
C                      yes; update old name, which will be copied
                       TEXT(K+1)=FILES(K,N)
                       GO TO 9
                    ELSE
C                      the names really are different
                       GO TO 7
                    ENDIF
                 ENDIF
    8         CONTINUE
C
C             the filename is in the directory and in =files.old
    9         FOUND(N)=.TRUE.
C
C             save the name for checking duplicates
              NLST=NLST+1
              DO 10 K=1,24
                    LIST(K,NLST)=TEXT(K+1)
   10         CONTINUE
C
C             copy the =files.old line to =files
              GO TO 6
    7      CONTINUE
C
C          this file is no longer present; append its name to GONE
           NOREM=NOREM+1
           IF(KREM.EQ.GLIMIT-3) GO TO 11
           KREM=KREM+1
           GONE(KREM)=' '
           DO 12 K=1,24
                 IF(TEXT(K+1).EQ.' ') GO TO 11
                 IF(KREM.EQ.GLIMIT-3) GO TO 11
                 KREM=KREM+1
                 GONE(KREM)=TEXT(K+1)
   12      CONTINUE
C          and omit this =files.old line from =files
           GO TO 11
C
C          write the output line in the new =files file
    6      WRITE(9,902) (TEXT(K),K=1,L)
      GO TO 11
C
C     list the unknown new files at the end of the updated list
    5 FIRST=.TRUE.
      DO 13 N=1,NCAT
C          if the file was already in the list don't list it again
           IF(FOUND(N)) GO TO 13
C
C          write heading if this is the first one
           IF(.NOT.FIRST) GO TO 14
           FIRST=.FALSE.
           WRITE(9,904) NOW
  904      FORMAT(' '/'New files unknown as of ',22A1/68('-'))
C
C          list the file as unknown
   14      TEXT(1)=' '
           DO 15 K=1,24
                 TEXT(1+K)=FILES(K,N)
   15      CONTINUE
           TEXT(26)=' '
           TEXT(27)='.'
           WRITE(9,902) (TEXT(K),K=1,27)
           NONEW=NONEW+1
   13 CONTINUE
C
C     check for duplicate names in =files
      IF(NLST.LT.2) GO TO 16
      NLM=NLST-1
      DO 17 N=1,NLM
            NP=N+1
            DO 18 NN=NP,NLST
                  DO 19 K=1,24
                        IF(LIST(K,N).NE.LIST(K,NN)) GO TO 18
   19             CONTINUE
                  WRITE(6,905) (LIST(K,N),K=1,24)
  905             FORMAT('filename "',24A1,'" is repeated')
                  NOREM=NOREM+1
                  GO TO 17
   18       CONTINUE
   17 CONTINUE
C
C     remove =files.old
   16 RC=SYSTEM('/bin/mv =files.old /tmp/=files.old')
      IF(NOREM+NONEW.EQ.0) THEN
         WRITE(0,906)
  906    FORMAT('files unchanged')
      ENDIF
      IF(NONEW.GT.0) THEN
         WRITE(0,907) NONEW
  907    FORMAT('new:',I3)
      ENDIF
      IF(NOREM.GT.0) THEN
         IF(KREM.EQ.GLIMIT-3) KREM=GLIMIT
         WRITE(0,908) NOREM,WHEN,NOW
  908    FORMAT('gone:',I3,' between [',22A1,'] and [',22A1,']')
         WRITE(0,909) (GONE(K),K=1,KREM)
  909    FORMAT((T9,78A1))
      ENDIF
      STOP
      END
