C
Code by Michael Kupferschmid
C
C     This program reads a filename from standard in and writes its
C     0, 1, 2, or 3-character extension on standard out.  The return
C     code is set to the length of the extension.
C
C     variable  meaning
C     --------  -------
C     EXIT      system routine stops program with a return code
C     FCHAR     FNAME as separate characters
C     FNAME     the filename as a 24-character word
C     K         index on the characters of FCHAR
C     KDOT      index in FCHAR where rightmost dot is found, or 1
C     L         length of filename
C     LENGTH    function returns index of last nonblank in a string
C
      CHARACTER*24 FNAME
      CHARACTER*1 FCHAR(24)
      EQUIVALENCE(FNAME,FCHAR)
C
C ------------------------------------------------------------------
C
C     receive the filename from standard in (typically a pipeline)
      READ(5,901,END=1) FNAME
  901 FORMAT(A24)
      L=LENGTH(FNAME,24)
      IF(L.GT.0) GO TO 2
C     no filename was given
    1 CALL EXIT(-1)
C
C     find the index in the filename of its rightmost dot
    2 DO 3 K=L,1,-1
           KDOT=K
           IF(FCHAR(K).EQ.'.') GO TO 4
    3 CONTINUE
C     the filename does not have an extension; output a null
      WRITE(6,*)
      CALL EXIT(0)
C
C     how many characters are after the rightmost dot?
    4 IF(L-KDOT.GT.3) THEN
C        the string after the rightmost dot is part of the filename
         WRITE(6,*)
         CALL EXIT(0)
      ENDIF
C
C     output the extension
      WRITE(6,902) (FCHAR(K),K=KDOT+1,L)
  902 FORMAT(24A1)
C
C     and set the return code to indicate its length
      CALL EXIT(L-KDOT)
      END
