C
Code by Michael Kupferschmid
C
C     just [LEFT {RIGHT]]
C
C     This program reads text from standard-in and writes it to
C     standard-out justified in columns LEFT through RIGHT.  If not
C     specified they are taken to be 1 and 68; if one parameter is
C     given it is taken to be LEFT and RIGHT is taken to be 68; if
C     two parameters are given the first is LEFT and the second is
C     RIGHT.
C
C     variable  meaning
C     --------  -------
C     EOS       T => this token is the last in a sentence
C     EXIT      system routine stops program with a return code
C     GETCOL    routine gets the column settings to use
C     IN        an input line
C     K         index on the characters of a string
C     LEFT      left margin
C     LENGTH    function returns index of last nonblank in a string
C     LIN       length of IN
C     LOUT      length of OUT
C     LQTK      length of a token excluding a trailing "
C     LTOK      length of a token
C     OUT       an output line
C     PUTOUT    routine writes an output line and starts the next
C     RC        unused; return code from SHFTKN
C     RIGHT     right margin
C     SHFTKN    routine shifts a string left by one token
C     SPACEL    routine removes leading and extra blanks
C
      CHARACTER*1 IN(1024),OUT(1024)
      INTEGER*4 RC,RIGHT
      LOGICAL*4 EOS
C
C ------------------------------------------------------------------
C
C     set the starting and ending columns
      CALL GETCOL(LEFT,RIGHT)
C
C     put blanks in the output line before column LEFT
      LOUT=0
      DO 1 K=1,LEFT-1
           LOUT=LOUT+1
           OUT(LOUT)='  '
    1 CONTINUE
C
C     read the next input line
    6 READ(5,901,END=2) IN
  901 FORMAT(1024A1)
C
C     remove leading and extra blanks
      CALL SPACEL(IN,1024)
      LIN=LENGTH(IN,1024)
C
C     a blank line signals the end of a paragraph
      IF(LIN.EQ.0) THEN
         CALL PUTOUT(OUT,LOUT,LEFT)
         WRITE(6,901)
         GO TO 6
      ENDIF
C
C     find the first blank to determine the length of the next token
    7 LTOK=0
      DO 3 K=1,LIN
           IF(IN(K).EQ.' ') GO TO 4
           LTOK=LTOK+1
    3 CONTINUE 
C
C     is it the end of a sentence?
    4 LQTK=LTOK
      IF(IN(LTOK).EQ.'"' .OR. IN(LTOK).EQ.")") LQTK=LTOK-1
      IF(LQTK.EQ.0) GO TO 8
      EOS=.FALSE.
      IF(IN(LQTK).EQ.'!'.OR.IN(LQTK).EQ.'?') EOS=.TRUE.
      IF(IN(LQTK).EQ.'.') THEN
         IF(LQTK.GE.3) THEN
C           check for _X. which is an initial
            IF(IN(LQTK-2).NE.' ') EOS=.TRUE.
         ENDIF
      ENDIF
C
C     will the token fit at all?
    8 IF(LTOK .GT. RIGHT-LEFT+1) THEN
C        no; write the line being assembled and then this token
         CALL PUTOUT(OUT,LOUT,LEFT)
         WRITE(6,901) (IN(K),K=1,LTOK)
C
C        shift out the token
         CALL SHFTKN(IN,LIN,1,RC)
C
C        get the next token, or the next input line
         LIN=LENGTH(IN,1024)
         IF(LIN.GT.0) GO TO 7
         GO TO 6
      ENDIF
C
C     will the token fit in this output line?
      IF(LOUT.NE.0 .AND. LOUT.GE.LEFT) THEN
         LOUT=LOUT+1
         OUT(LOUT)=' '
      ENDIF
      IF(LOUT+LTOK .LE.RIGHT) THEN
C        yes; copy it into the output line
         DO 5 K=1,LTOK
              LOUT=LOUT+1
              OUT(LOUT)=IN(K) 
    5    CONTINUE
         IF(EOS) THEN
              LOUT=LOUT+1
              OUT(LOUT)=' '
         ENDIF
C
C        shift out the token
         CALL SHFTKN(IN,LIN,1,RC)
C
C        get the next token, or the next input line
         LIN=LENGTH(IN,1024)
         IF(LIN.GT.0) GO TO 7
         GO TO 6
      ENDIF
C
C     no; write the output line and start the next
      CALL PUTOUT(OUT,LOUT,LEFT)
      GO TO 7
C
C     found the end of the input
    2 CALL PUTOUT(OUT,LOUT,LEFT)
      CALL EXIT(0)
      END
C
      SUBROUTINE GETCOL(LEFT,RIGHT)
C     This routine gets the column range to use.
C
C     variable  meaning
C     --------  -------
C     ARG       a run-line parameter
C     DTB       routine converts numerals to an INTEGER*4
C     EXIT      system routine stops program with a return code
C     GETARG    system routine gets one command-line argument
C     IARGC     system function for number of command-line arguments
C     LEFT      column where text starts
C     NARG      number of run-line arguments
C     RC        return code; 0 => all went well
C     RIGHT     column beyond which text does not extend
C     SHIFTR    routine makes string flush right in its length
C
      INTEGER*4 RIGHT,RC
      CHARACTER*4 ARG
C
C ------------------------------------------------------------------
C
C     set the column range from the parameters
      NARG=IARGC()
      IF(NARG.EQ.0) THEN
         LEFT=1
         RIGHT=68
         RETURN
      ENDIF
      IF(NARG.EQ.1) THEN
         RIGHT=68
         CALL GETARG(1,ARG)
         CALL SHIFTR(ARG,4)
         CALL DTB(ARG,4, LEFT,RC)
         IF(RC.NE.0) GO TO 1
         IF(LEFT.LE.0) GO TO 1
         RETURN
      ENDIF
      IF(NARG.EQ.2) THEN
         CALL GETARG(1,ARG)
         CALL SHIFTR(ARG,4)
         CALL DTB(ARG,4, LEFT,RC)
         IF(RC.NE.0) GO TO 1
         IF(LEFT.LE.0) GO TO 1
         CALL GETARG(2,ARG)
         CALL SHIFTR(ARG,4)
         CALL DTB(ARG,4, RIGHT,RC)
         IF(RC.NE.0) GO TO 1
         IF(RIGHT.LE.LEFT) GO TO 1
         RETURN
      ENDIF
C
C     handle parameter errors
    1 WRITE(0,901)
  901 FORMAT('usage: just [LEFT [RIGHT}] where RIGHT > LEFT > 0')
      CALL EXIT(1)
      END
C
      SUBROUTINE PUTOUT(OUT,LOUT,LEFT)
C     This routine writes an output line and starts the next one.
C
C     variable  meaning
C     --------  -------
C     K         index on the characters of a string
C     LEFT      column where text starts
C     LENGTH    function returns index of last nonblank in a string
C     LOUT      length of output line
C     OUT       output line
C
      CHARACTER*1 OUT(*)
C
C ------------------------------------------------------------------
C
      LOUT=LENGTH(OUT,LOUT)
      IF(LOUT.EQ.0) RETURN
      WRITE(6,901) (OUT(K),K=1,LOUT)
  901 FORMAT(1024A1)
      LOUT=LEFT-1
      RETURN
      END
