C
C     This program compares two text files and reports lines in each
C     that are not in the other.
C
C     unit  use
C     ----  ---
C       0   error message if either file is too long
C       1   first input file
C       2   second input file
C       3   output file containing lines in one but not the other
C       5   filename inputs from the keyboard
C       6   message reporting line counts
C
C      $?   meaning
C     ----  -------
C       0   no difference was found between the input files
C       1   one file has lines that are not in the other
C       2   each file has lines that are not in the other
C       3   an input file had too many lines
C
C     variable  meaning
C     --------  -------
C     CMP       routine writes out lines that differ
C     EXIT      system routine stops program with a return code
C     FILE1     the bytes of file 1
C     FILE2     the bytes of file 2
C     GETFIL    routine attaches a file for a given purpose
C     INP       routine reads in lines from the files
C     K         index on the characters of TITLE
C     LENGTH    function returns index of last nonblank in a string
C     LN1       length of NAME1
C     LN2       length of NAME2
C     LT        index of last nonblank in TITLE
C     N1        number of lines in file 1
C     N2        number of lines in file 2
C     NAME1     name of file attached to unit 1
C     NAME2     name of file attached to unit 2
C     NMAX      limit on N1 or N2
C     RC12      1 => a line in 1 is not in 2
C     RC21      1 => a line in 2 is not in 1
C     SPACEL    routine removes leading and extra blanks
C     STRINS    routine inserts one character string into another
C     TITLE     heading in output file
C     TMPLT     template for an ouput title
C
C     prepare to store the files
      PARAMETER(NMAX=1000)
      CHARACTER*1 FILE1(256,NMAX),FILE2(256,NMAX)
C
C     prepare to compare the files
      CHARACTER*24 NAME1,NAME2
      INTEGER*4 RC12,RC21
C
C     prepare to report the differences
      CHARACTER*1 TMPLT(75)/'l','i','n','e','s',' ','i','n',' ',
     ;           24*' ',' ','t','h','a','t',' ','a','r','e',' ',
     ;                      'n','o','t',' ','i','n',' ',
     ;           24*' ',':'/
      CHARACTER*1 TITLE(75)
C
C ------------------------------------------------------------------
C
C     read the files into memory
      CALL GETFIL('file 1',6,' ',0,1,1)
      INQUIRE(UNIT=1,NAME=NAME1)
      CALL INP(1,FILE1,NAME1,NMAX, N1)
      CALL GETFIL('file 2',6,' ',0,2,1)
      INQUIRE(UNIT=2,NAME=NAME2)
      CALL INP(2,FILE2,NAME2,NMAX, N2)
C
C     report the file lengths and differences
      WRITE(6,901) N1,NAME1,N2,NAME2
  901 FORMAT('  there are ',I4,' lines in ',A24/
     ;       '  there are ' I4,' lines in ',A24)
      CALL GETFIL('differences',11,' ',0,3,2)
C
C     report lines in file 1 that are not in file 2
      LN1=LENGTH(NAME1,24)
      LN2=LENGTH(NAME2,24)
      CALL STRINS(TMPLT,75,NAME1,LN1,10,0, TITLE,LT)
      CALL STRINS(TITLE,75,NAME2,LN2,51,2, TITLE,LT)
      CALL SPACEL(TITLE,LT)
      LT=LENGTH(TITLE,LT)
      WRITE(3,902) (TITLE(K),K=1,LT)
  902 FORMAT(75A1)
      CALL CMP(FILE1,N1,FILE2,N2,RC12)
      WRITE(3,903)
  903 FORMAT(' ')
C
C     report lines in file 2 that are not in file 1
      CALL STRINS(TMPLT,75,NAME2,LN2,10,0, TITLE,LT)
      CALL STRINS(TITLE,75,NAME1,LN1,51,2, TITLE,LT)
      CALL SPACEL(TITLE,LT)
      LT=LENGTH(TITLE,LT)
      WRITE(3,902) (TITLE(K),K=1,LT)
      CALL CMP(FILE2,N2,FILE1,N1,RC21)
C
C     set the return code
      CALL EXIT(RC12+RC21)
      END
C
      SUBROUTINE INP(NUNIT,FILE,NAME,NMAX, I)
C     This routine reads the file on unit NUNIT into the array FILE.
C
C     variable  meaning
C     --------  -------
C     EXIT      system routine stops program with a return code
C     FILE      array containing the bytes of the file
C     I         number of lines found in the file
C     K         index on the characters of a line
C     L         length of a line
C     LENGTH    function returns index of last nonblank in a string
C     LINE      a line read from a file
C     NAME      the name of the file attached to NUNIT
C     NMAX      limit on the number of lines read
C     NUNIT     unit number from which to read input lines
C     TEXT      LINE as single characters
C
C     formal parameters
      CHARACTER*1 FILE(256,NMAX)
      CHARACTER*24 NAME
C
C     prepare to read lines from the file
      CHARACTER*256 LINE
      CHARACTER*1 TEXT(256)
      EQUIVALENCE(LINE,TEXT)
C
C ------------------------------------------------------------------
C
C     copy file lines into array columns
      I=0
    3 READ(NUNIT,901,END=1) LINE
  901 FORMAT(A256)
      I=I+1
      IF(I.GT.NMAX) THEN
         WRITE(0,991) NMAX,NAME
  991    FORMAT('more than ',I4,' lines in file ',A24)
         CALL EXIT(3)
      ENDIF
      L=LENGTH(TEXT,256)
      DO 2 K=1,256
           IF(K.LE.L) THEN
              FILE(K,I)=TEXT(K)
           ELSE
              FILE(K,I)=' '
           ENDIF
    2 CONTINUE
      GO TO 3
    1 RETURN
      END
C
      SUBROUTINE CMP(FILEA,LA,FILEB,LB,RC)
C     This routine searches FILEB for each line in FILEA
C     and writes any that are not found on unit 3.
C
C     variable  meaning
C     --------  -------
C     FILEA     one file
C     FILEB     the other file
C     K         index on the characters of TEXT
C     KA        index on the lines in the A file
C     KB        index on the lines in the B file
C     L         length of a line being written
C     LA        number of lines in the A file
C     LB        number of lines in the B file
C     LENGTH    function returns index of last nonblank in a string
C     LINE      a file line written out
C     RC        return code; 0 => no differences found
C     TEXT      LINE as single characters
C
C     formal parameters
      CHARACTER*256 FILEA(LA),FILEB(LB)
      INTEGER*4 RC
C
C     local variables
      CHARACTER*256 LINE
      CHARACTER*1 TEXT(256)
      EQUIVALENCE(LINE,TEXT)
C
C ------------------------------------------------------------------
C
C     assume no discrepancy will be found
      RC=0
C
C     for each line in the A file
      DO 1 KA=1,LA
C          search for it in the B file
           DO 2 KB=1,LB
                IF(FILEB(KB).EQ.FILEA(KA)) GO TO 1
    2      CONTINUE
           RC=1
C
C          this line of the A file is not in the B file
           LINE=FILEA(KA)
           L=LENGTH(TEXT,256)
           WRITE(3,901) KA,(TEXT(K),K=1,L)
  901      FORMAT(I4,1X,256A1)
    1 CONTINUE
      RETURN
      END
