C
Code by Michael Kupferschmid
C
C     This program writes a message from a Makefile.
C     usage: makemsg "advisory message"
C        or: echo "error message" | makemsg
C        or: cat fyle | makemsg
C
C     If the message string is given as a parameter it is assumed
C     to be advisory and the program stops with return code 0.
C
C     If the message is found on standard-in it is assumed
C     to be an error report and the program stops with return code 1
C     to stop make.
C
C     variable  meaning
C     --------  -------
C     CMSG      command-line argument as separate characters
C     EXIT      system routine stops program with a return code
C     GETARG    system routine gets one command-line argument
C     IARGC     system function for number of command-line arguments
C     K         index on the characters of CARG
C     L         index of last nonblank in ARG
C     LENGTH    function returns index of last nonblank in a string
C     MSG       message to write
C     NARG      number of command-line arguments found
C     RC        return code; 0 => no error message was found
C
      CHARACTER*256 MSG
      CHARACTER*1 CMSG(256)
      EQUIVALENCE(MSG,CMSG)
      INTEGER*4 RC
C
C ------------------------------------------------------------------
C
C     the message, if any, will be written on /dev/tty
      OPEN(UNIT=0,FILE='/dev/tty')
      RC=0
C
C     if there is a parameter it is an advisory message
      NARG=IARGC()
      IF(NARG.EQ.0) GO TO 1
      CALL GETARG(1,MSG)
      L=LENGTH(MSG,256)
      IF(L.EQ.0) GO TO 1
      WRITE(0,901) (CMSG(K),K=1,L)
  901 FORMAT(256A1)
      GO TO 2
C
C     if there is no parameter assume input on standard in
    1 READ(5,900,END=2) MSG
  900 FORMAT(A256)
      L=LENGTH(MSG,256)
      IF(L.EQ.0) GO TO 2
      RC=1
      WRITE(0,901) (CMSG(K),K=1,L)
      GO TO 1
C
C     stop the program
    2 CALL EXIT(RC)
      END
