C
Code by Michael Kupferschmid
C
C     This program reads a file of LaTeX source text, removes the
C     markups, and writes a file containing what is left.
C
C     variable  meaning
C     --------  -------
C     BLANK     T => blanking something out
C     BNKALL    routine removes a command and its {argument}
C     CHAR      abcdefghijklmnopqrstuvwxyz'
C     FROM      characters to translate from
C     INTO      corresponding characters to translate into
C     J         index on characters in FROM and INTO
C     K         index on the characters of LINE
C     KK        second index on the characters of LINE
C     KKK       third index on the characters of LINE
C     L         length of LINE
C     LENGTH    function returns index of last nonblank in a string
C     LINE      a line of text read, processed, and written
C     LL        maximum length of transliteration
C     M         index on the characters of CHAR
C     MIN0      Fortran function returns smaller of INTEGER*4s
C     NC        number of characters in translate table
C     SPACEL    routine removes leading and extra blanks

C     prepare to read and write lines of text
      CHARACTER*1 LINE(256)
      LOGICAL*4 BLANK
C
C     prepare the translation table
      PARAMETER(NC=67)
      CHARACTER*1 FROM(NC)/'{','}',',','.','(',')',';',':',"'",'"',
     ;                     '-','+','=','1','2','3','4','5','6','7',
     ;                     '8','9','0','!','?','@','#','$','%','^',
     ;                     '&','*','_','|','/','>','<',']','[','~',
     ;                     '`','A','B','C','D','E','F','G','H','I',
     ;                     'J','K','L','M','N','O','P','Q','R','S',
     ;                     'T','U','V','W','X','Y','Z'/
      CHARACTER*1 INTO(NC)/' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
     ;                     ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
     ;                     ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
     ;                     ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
     ;                     ' ','a','b','c','d','e','f','g','h','i',
     ;                     'j','k','l','m','n','o','p','q','r','s',
     ;                     't','u','v','w','x','y','z'/
C
C     characters legal in a transliteration
      CHARACTER*1 CHAR(27)/'a','b','c','d','e','f','g','h','i','j',
     ;                     'k','l','m','n','o','p','q','r','s','t',
     ;                     'u','v','w','x','y','z',"'"/
C
C ------------------------------------------------------------------
C
C     read the next input line
    2 READ(5,900,END=1) LINE
  900 FORMAT(256A1)
      L=LENGTH(LINE,256)
      IF(L.EQ.0) GO TO 2
C
C     blank out comments
      IF(LINE(1).EQ.'%') GO TO 2
      BLANK=.FALSE.
      DO 3 K=1,L
           IF(LINE(K).EQ.'%') BLANK=.TRUE.
           IF(BLANK) LINE(K)=' '
    3 CONTINUE
      L=LENGTH(LINE,L)
      IF(L.EQ.0) GO TO 2
C
C     remove <transliterations> up to 16 characters long
      KSTART=1
    8 DO 4 K=KSTART,L
C          look for the start of a transliteration
           IF(LINE(K).NE.'<') GO TO 4
C
C          this might be one; look for an ending >
           LL=MIN0(L-K+1,18)
           DO 5 KK=K+1,K+LL
                IF(LINE(KK).EQ.'>') THEN
C                  found the end of a transliteration; blank it
                   DO 6 KKK=K,KK
                        LINE(KKK)=' '
    6              CONTINUE
                   IF(KK.EQ.L) GO TO 7
                   KSTART=KK+1
                   GO TO 8
                ELSE
                   DO 9 M=1,27
                        IF(LINE(KK).EQ.CHAR(M)) GO TO 5
    9              CONTINUE
C                  character not legal in a transliteration
                   KSTART=KK
                   GO TO 8
                ENDIF
    5      CONTINUE
    4 CONTINUE
C
C     blank out certain LaTeX commands with their arguments
    7 CALL BNKALL(LINE,L,'\documentclass',14,1)
      CALL BNKALL(LINE,L,'\usepackage',11,1)
      CALL BNKALL(LINE,L,'\pagestyle',10,1)
      CALL BNKALL(LINE,L,'\vspace',7,1)
      CALL BNKALL(LINE,L,'\hspace',7,1)
      CALL BNKALL(LINE,L,'\begin{tabular}',15,1)
      CALL BNKALL(LINE,L,'\begin',6,1)
      CALL BNKALL(LINE,L,'\end',4,1)
      CALL BNKALL(LINE,L,'\label',6,1)
      CALL BNKALL(LINE,L,'\ref',4,1)
      CALL BNKALL(LINE,L,'\input',6,1)
      CALL BNKALL(LINE,L,'\S{',3,1)
      CALL BNKALL(LINE,L,'\site{',6,2)
      CALL BNKALL(LINE,L,'\hebrew{',8,1)
      CALL BNKALL(LINE,L,'\chiriq{',8,1)
      CALL BNKALL(LINE,L,'\tsere{',7,1)
      CALL BNKALL(LINE,L,'\segol{',7,1)
      CALL BNKALL(LINE,L,'\halfsegol{',11,1)
      CALL BNKALL(LINE,L,'\patach{',8,1)
      CALL BNKALL(LINE,L,'\halfpatach{',12,1)
      CALL BNKALL(LINE,L,'\qamats{',8,1)
      CALL BNKALL(LINE,L,'\halfqamats{',12,1)
      CALL BNKALL(LINE,L,'\awe{',5,1)
      CALL BNKALL(LINE,L,'\qubbuts{',9,1)
      CALL BNKALL(LINE,L,'\sheva{',7,1)
      CALL BNKALL(LINE,L,'\lcholam{',9,1)
      CALL BNKALL(LINE,L,'\rcholam{',9,1)
      CALL BNKALL(LINE,L,'\dagesh{',8,2)
      CALL BNKALL(LINE,L,'\meteg{',7,2)
      CALL BNKALL(LINE,L,'\hebspc{',8,2)
C
C     finish processing the LaTeX commands
      BLANK=.FALSE.
      DO 10 K=1,L
C           remove remaining LaTeX \command names
            IF(BLANK) THEN
               IF(LINE(K).EQ.' ' .OR.
     ;            LINE(K).EQ.'{' .OR.
     ;            LINE(K).EQ.'[') THEN
                  BLANK=.FALSE.
               ENDIF
               LINE(K)=' '
               GO TO 10
            ENDIF
            IF(.NOT.BLANK) THEN
               IF(LINE(K).EQ.'\') THEN
                  BLANK=.TRUE.
                  LINE(K)=' '
                  GO TO 10
               ENDIF
            ENDIF
C
C           translate the remaining characters
            DO 11 J=1,NC-26
                  IF(LINE(K).NE.FROM(J)) GO TO 11
                  LINE(K)=INTO(J)
                  GO TO 10
   11       CONTINUE
   10 CONTINUE
C
C     write the stripped line
      CALL SPACEL(LINE,L)
      L=LENGTH(LINE,256)
      IF(L.GT.1) WRITE(6,900) (LINE(K),K=1,L)
      GO TO 2
C
C     the end of the input file has been reached
    1 STOP
      END
C
      SUBROUTINE BNKALL(LINE,L,CMD,LC,NA)
C     This routine blanks out the given command
C     along with its NA arguments.
C
C     variable  meaning
C     --------  -------
C     CMD       command to be removed
C     DONE      the end of the command argument has been found
C     FINDST    routine finds starting location of a substring
C     IA        number of arguments removed so far
C     K         index on the characters in LINE
C     KSTART    index in LINE of beginning of CMD string
C     L         length of LINE
C     LC        length of CMD
C     LINE      the line of text to be processed
C     NA        number of arguments to remove
C
      CHARACTER*1 LINE(L),CMD(LC)
      LOGICAL*4 DONE
C
C ------------------------------------------------------------------
C
C     is the command in this input line?
    2 CALL FINDST(LINE,L,CMD,LC, KSTART)
      IF(KSTART.LE.0) RETURN
C
C     yes; remove the command name
      DO 1 K=KSTART,KSTART+LC-1
           LINE(K)=' '
    1 CONTINUE
C
C     then remove the command parameter
      IA=0
      DONE=.FALSE.
      K=KSTART+LC-1
    3 K=K+1
      IF(K.GT.L) RETURN
      IF(LINE(K).EQ.'}') THEN
         IA=IA+1
         IF(IA.EQ.NA) DONE=.TRUE.
      ENDIF
      LINE(K)=' '
      IF(DONE) GO TO 2
      GO TO 3
      END
