C
Code by Michael Kupferschmid
C
C     This program finds repeated words in text.
C
C     variable  meaning
C     --------  -------
C     DEBUG     T => write debugging output
C     INTOK     T => we are in a token
C     J         index on punctuation characters
C     K         index on the characters of a string
C     L         length of remaining input line
C     LENGTH    function returns index of last nonblank in a string
C     LNUM      number of line being processed
C     LOCASE    function returns upper case for mixed
C     LP        length of previous token
C     LT        length of current token
C     MAX0      Fortran function for larger of INTEGER*4s
C     PREV      previous token
C     PUNCT     punctuation characters
C     SAME      T => a word is repeated
C     SHIFTL    routine removes leading blanks from a string
C     TEXT      input line
C     TOKEN     current token
C
      CHARACTER*1 TEXT(80),TOKEN(80),PREV(80)/80*' '/,LOCASE
      CHARACTER*1 PUNCT(42)/'1','2','3','4','5','6','7','8','9','0',
     ;                      '.',',',':',';','"',"'",'/','\\','|',
     ;                      '{','}','[',']','(',')','+','-','=','_',
     ;                      '~','`','!','@','#','$','%','^','&','*',
     ;                      '?','<','>'/
      LOGICAL*4 SAME,DEBUG/.FALSE./,INTOK
C
C ------------------------------------------------------------------
C
C     to begin with there is no previous token
      LP=0
C
C     read the next line of text
      LNUM=0
    2 READ(5,901,END=1) TEXT
  901 FORMAT(80A1)
      LNUM=LNUM+1
C
C     remove punctuation
      L=LENGTH(TEXT,80)
      DO 7 K=1,L
           DO 8 J=1,42
                IF(TEXT(K).EQ.PUNCT(J)) TEXT(K)=' '
    8      CONTINUE
           TEXT(K)=LOCASE(TEXT(K))
    7 CONTINUE
C
C     remove leading blanks
    6 IF(DEBUG) WRITE(6,991) LNUM,(TEXT(K),K=1,L)
  991 FORMAT(I6,1X,80A1)
      CALL SHIFTL(TEXT,80)
      L=LENGTH(TEXT,80)
      IF(L.EQ.0) GO TO 2
C
C     extract the next token, blanking it out in TEXT
      INTOK=.TRUE.
      DO 3 K=1,80
           IF(INTOK) THEN
              IF(K.LE.L .AND. TEXT(K).NE.' ') THEN
                 TOKEN(K)=TEXT(K)
                 LT=K
                 TEXT(K)=' '
              ELSE
                 INTOK=.FALSE.
                 TOKEN(K)=' '
              ENDIF
           ELSE
              TOKEN(K)=' '
           ENDIF
    3 CONTINUE
C
C     compare this word to the previous word
    4 IF(DEBUG) WRITE(6,992) LT,TOKEN
  992 FORMAT('LT=',I2,' TOKEN=',80A1)
      IF(DEBUG) WRITE(6,993) LP,PREV
  993 FORMAT('LP=',I2,' PREV=',80A1)
      SAME=.TRUE.
      DO 5 K=1,80
           IF(K.LE.MAX0(LT,LP) .AND. TOKEN(K).NE.PREV(K))
     ;        SAME=.FALSE.
C
C          save the current token as the previous for next time
           IF(K.LE.LT) THEN
              PREV(K)=TOKEN(K)
           ELSE
              PREV(K)=' ' 
           ENDIF 
    5 CONTINUE 
C
C     report words that are the same
      IF(SAME) THEN
         WRITE(6,902) LNUM,(TOKEN(K),K=1,LT)
  902    FORMAT(I6,1X,80A1)
         IF(DEBUG) STOP
      ENDIF
      LP=LT
      GO TO 6
C
C     the end of the file has been reached
    1 STOP
      END
